#include "states.h"

#include "state.h"
#include "station.h"
#include "sounds.h"
#include "text.h"

Startup::Startup() = default;

State* Startup::putDown(const Context context, Station* newStation)
{
    const int newStationIndex = context.getStationIndex(newStation);
    if (newStationIndex == 0)
    {
        return new WaitingForGameStart();
    }

    return this;
}

State* Startup::update(const Context context)
{
    return this;
}

String Startup::updateDisplay(const Context& context)
{
    return "Legg mi uf die  ersti Station!";
}

void Startup::activated(Context context)
{
}


//------------------------------------


WaitingForGameStart::WaitingForGameStart() = default;

State* WaitingForGameStart::pickedUp(const Context context)
{
    return new OnTheMove(context.stations + 1, context.stations); // first element of the array
}

State* WaitingForGameStart::update(const Context context)
{
    return this;
}

String WaitingForGameStart::updateDisplay(const Context& context)
{
    return WAITING_FOR_START_TEXT;
}

void WaitingForGameStart::activated(const Context context)
{
    context.dfPlayer->loop(WAITING_FOR_GAME_START);
}


//------------------------------------


OnTheMove::OnTheMove(Station* targetStation, Station* previousStation)
{
    this->targetStation = targetStation;
    this->previousStation = previousStation;
}

State* OnTheMove::putDown(const Context context, Station* newStation)
{
    if (newStation == this->targetStation)
    {
        if (targetStation == context.stations) // first station
        {
            return new End();
        }
        return new Hacking(this->targetStation);
    }

    return new IncorrectStation(targetStation, previousStation);
}

State* OnTheMove::update(const Context context)
{
    return this;
}

String OnTheMove::updateDisplay(const Context& context)
{
    const int index = context.getStationIndex(this->targetStation);
    if (index == 0)
    {
        return TOBIONE_AFTER_TEXT;
    }
    else if (index == 1)
    {
        return MAIN_AFTER_PICKUP_TEXT;
    }
    else if (index == 2)
    {
        return MUHVELLA_AFTER_TEXT;
    }
    else if (index == 3)
    {
        return MAGIE_AFTER_TEXT;
    }
    else if (index == 4)
    {
        return BICOLA_AFTER_TEXT;
    }
    
    return "UNDEFINED";
}

void OnTheMove::activated(const Context context)
{
    const int index = context.getStationIndex(this->targetStation);
    if (index == 0)
    {
        context.dfPlayer->play(AFTER_TOBIONE);
    }
    else if (index == 1)
    {
        context.dfPlayer->play(AFTER_MAIN_STATION);
    }
    else if (index == 2)
    {
        context.dfPlayer->play(AFTER_MUHVELLA);
    }
    else if (index == 3)
    {
        context.dfPlayer->play(AFTER_MAGIE);
    }
    else if (index == 4)
    {
        context.dfPlayer->play(AFTER_BICOLA);
    }
}


//------------------------------------


IncorrectStation::IncorrectStation(Station* targetStation, Station* previousStation)
{
    this->targetStation = targetStation;
    this->previousStation = previousStation;
}

State* IncorrectStation::pickedUp(const Context context)
{
    return new OnTheMove(this->targetStation, previousStation);
}

State* IncorrectStation::update(const Context context)
{
    return this;
}

String IncorrectStation::updateDisplay(const Context& context)
{
    return WRONG_STATION_TEXT;
}

void IncorrectStation::activated(const Context context)
{
    context.dfPlayer->play(WRONG_STATION);
}


//------------------------------------


Hacking::Hacking(Station* currentStation): timer(5000, false)
{
    this->currentStation = currentStation;
    this->timer.start();
}

State* Hacking::pickedUp(const Context context)
{
    return new Complain(currentStation);
}

State* Hacking::update(const Context context)
{
    Serial.println(String(context.delta));
    const bool done = timer.update(context.delta);

    if (done)
    {
        const int currentStationIndex = context.getStationIndex(currentStation);
        const int nextStationIndex = (currentStationIndex + 1) % context.stationCount;
        Station* nextStation = context.stations + nextStationIndex;
        return new WaitingForPickup(currentStation, nextStation);
    }
    return this;
}

String Hacking::updateDisplay(const Context& context)
{
    auto currentStationIndex = context.getStationIndex(this->currentStation);
    if (currentStationIndex == 1)
    {
        return MUHVELLA_HACKING_TEXT;
    }
    if (currentStationIndex == 2)
    {
        return MAGIE_HACKING_TEXT;
    }
    if (currentStationIndex == 3)
    {
        return BICOLA_HACKING_TEXT;
    }
    if (currentStationIndex == 4)
    {
        return TOBIONE_HACKING_TEXT;
    }
    return "UNDEFINED";
}

void Hacking::activated(const Context context)
{
    auto currentStationIndex = context.getStationIndex(this->currentStation);
    if (currentStationIndex == 1)
    {
        timer = Timer(25000, false);
        context.dfPlayer->play(HACKING_STATION_MUHVELLA);
    }
    if (currentStationIndex == 2)
    {
        timer = Timer(29000, false);
        context.dfPlayer->play(HACKING_STATION_MAGIE);
    }
    if (currentStationIndex == 3)
    {
        timer = Timer(29000, false);
        context.dfPlayer->play(HACKING_STATION_BICOLA);
    }
    if (currentStationIndex == 4)
    {
        timer = Timer(40000, false);
        context.dfPlayer->play(HACKING_STATION_TOBIONE);
    }
    timer.start();
}


//------------------------------------


Complain::Complain(Station* currentStation)
{
    this->currentStation = currentStation;
}

State* Complain::putDown(const Context context, Station* newStation)
{
    if (this->currentStation == newStation) // was put back on correct station
    {
        return new Hacking(this->currentStation);
    }
    return this; // was not put back on correct station (just keeps complaining)
}

State* Complain::update(Context context)
{
    return this;
}

String Complain::updateDisplay(const Context& context)
{
    return COMPLAIN_TEXT;
}

void Complain::activated(Context context)
{
}


//------------------------------------


WaitingForPickup::WaitingForPickup(Station* currentStation, Station* targetStation)
{
    this->currentStation = currentStation;
    this->targetStation = targetStation;
}

State* WaitingForPickup::pickedUp(Context context)
{
    return new OnTheMove(targetStation, currentStation);
}

State* WaitingForPickup::update(Context context)
{
    return this;
}

String WaitingForPickup::updateDisplay(const Context& context)
{
    auto currentStationIndex = context.getStationIndex(this->currentStation);
    if (currentStationIndex == 1)
    {
        return MUHVELLA_PICKUP_TEXT;
    }
    if (currentStationIndex == 2)
    {
        return MAGIE_PICKUP_TEXT;
    }
    if (currentStationIndex == 3)
    {
        return BICOLA_PICKUP_TEXT;
    }
    if (currentStationIndex == 4)
    {
        return TOBIONE_PICKUP_TEXT;
    }
}

void WaitingForPickup::activated(Context context)
{
    auto currentStationIndex = context.getStationIndex(this->currentStation);
    if (currentStationIndex == 1)
    {
        context.dfPlayer->loop(PICKUP_STATION_MUHVELLA);
    }
    if (currentStationIndex == 2)
    {
        context.dfPlayer->loop(PICKUP_STATION_MAGIE);
    }
    if (currentStationIndex == 3)
    {
        context.dfPlayer->loop(PICKUP_STATION_BICOLA);
    }
    if (currentStationIndex == 4)
    {
        context.dfPlayer->loop(PICKUP_STATION_TOBIONE);
    }
}


//------------------------------------


End::End(): timer(Timer(0, false))
{
}

State* End::pickedUp(const Context context)
{
    // todo complain
    return new Complain(context.stations);
}

State* End::update(const Context context)
{
    const bool done = timer.update(context.delta);
    if (done)
    {
        return new WaitingForGameStart();
    }
    return this;
}

String End::updateDisplay(const Context& context)
{
    return END_TEXT;
}

void End::activated(const Context context)
{
    context.dfPlayer->play(END);
    timer = Timer(38000, false);
    timer.start();
}
